/* ClientSocket.cpp

java-like implementation of sockets.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Peter Dikant <peter.dikant@mobilexag.de>

This file is distributed under the GPL, see file COPYING for details. */

#ifdef LINUX
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>
#include "socket.h"
#else
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <iostream>
#include <memory.h>
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#endif

using namespace std;

#include "ClientSocket.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#ifdef LINUX
static void Sleep(int useconds)
{
	usleep(useconds);
}
static void closesocket(int socket)
{
	close(socket);
}
#endif

/**
 * this constructor is used to initialize the class with an already
 * given socket. This is the situation when a server is listening on
 * 
 */
CClientSocket::CClientSocket(SOCKET client)
{
	this->client = client;
	this->host = NULL;
	this->port = -1;
} 

CClientSocket::CClientSocket(const char *host, int port) {
	this->client = INVALID_SOCKET;
	this->host = (char*)host;
	this->port = port;
}

/**
 * tidy up and close the client socket connection.
 */
CClientSocket::~CClientSocket()
{
	Sleep(500); // wait for slow java clients to close the socket
	closesocket(client);
}

/**
 * read a line from the socket with a predefined buffer size
 */
void CClientSocket::readLine(char *buffer)
{
	this->readLine(buffer, 1024);
} /* readLine */

int CClientSocket::getBlock(char *buffer, int size)
{
	return recv(client, buffer, size, 
#ifdef LINUX
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
		0
#endif
		);
}

/**
 * this method will read characters from the socket connection until a newline
 * has been reached. the method will only terminate on newline or buffer full.
 */
void CClientSocket::readLine(char *buffer, int bufferSize)
{
	int pos = 0;
	char* startofbuffer = buffer;

	while (pos < bufferSize) {
	
		int ret = recv(client, buffer, 1, 
#ifdef LINUX
# ifdef MSG_NOSIGNAL
		MSG_NOSIGNAL
# else
		0x4000
# endif
#else
			0
#endif
			);
		if (ret <= 0) { 
#ifndef LINUX
			if (WSAGetLastError() != 0) {
				cerr << "Error reading from socket: " << WSAGetLastError() << endl;
			}
#else
			printf ("Error reading from socket: ret=%d errno=%d\n", ret, errno);
#endif
			*buffer = '\0';
			return;
		}

		if (*buffer++ == '\n') {
			*--buffer = '\0'; // get rid of LF
			if (*--buffer == '\r') { // get rid of optional CR
				*buffer = '\0';
			}
//			printf("socket: %s\n", startofbuffer);
			return;
		}
	}
} /* readLine */

/**
 * print a line of text to the socket connection.
 */
int CClientSocket::print(const char *line)
{
/*	printf(">%s", line);
	if(line[strlen(line)-1]!='\n')
	  printf("\n");*/
#ifdef LINUX
	//write(client, line, strlen(line));
	int ret = send(client, line, strlen(line), 0);
	if (ret <= 0) {
		perror("send returned:");
		printf("send error: %d\n", ret);
		return -1;
	}
#else
	if (send(client, line, strlen(line), 0) == SOCKET_ERROR) {
		cerr << "Error writing to socket: " << WSAGetLastError() << endl;
		return -1;
	}
#endif
    return 0;
} /* print */

/**
 * this a print method that will accept a list of arguments to be included in the
 * output line, similar to the normal printf statement
 */
int CClientSocket::myprintf(const char *line,...)
{
	char buffer[1024];
	va_list arglist;
	va_start(arglist, line);
	vsprintf(buffer, line, arglist);
	va_end(arglist);
	return print(buffer);
} /* printf */

bool CClientSocket::establishConnection()
{

#ifdef LINUX
	client = connect_to_host(host, port);
	printf("port is %d\n", client);
	return true;
#else
	WSAData wsaData;
	int nRet;

	// Start Winsock up
	nRet = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (nRet != 0) {
		cerr << "WSAStartup() returned error code " << nRet << "." << endl;
		return false;
	}

	if (client != INVALID_SOCKET) {
		return false;
	}

	// convert host address to useable format
	u_long remoteAddr = inet_addr(host);
	if (remoteAddr == INADDR_NONE) {
		// pcHost isn't a dotted IP, so resolve it through DNS
        hostent* he = gethostbyname(host);
		if (he == 0) {
			fprintf(stderr, "Could not resolve remote server address: %s\n" ,host);
			return false;
		}
		remoteAddr = *((u_long*)he->h_addr_list[0]);
	}
	
	// open socket connection
	client = socket(AF_INET, SOCK_STREAM, 0);
	if (client == INVALID_SOCKET) {
		cerr << "Could not create client socket: " << WSAGetLastError() << endl;
		return false;
	}

	sockaddr_in sinRemote;
	sinRemote.sin_family = AF_INET;
	sinRemote.sin_addr.s_addr = remoteAddr;
	sinRemote.sin_port = htons(port);
	if (connect(client, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == SOCKET_ERROR) {
		cerr << "Could not connect to server: " << WSAGetLastError() << endl;
		return false;
	}
	return true;
#endif
} /* establishConnection */
