/* SimpleCapi.cpp

implementation of the SimpleCapi class. (See SimpleCapi.h)

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#ifdef LINUX
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "CapiHandler2.h"
#include "Log.h"
#include "SimpleCapi.h"
#include "wav2law.h"
#define EnterCriticalSection pthread_mutex_lock
#define LeaveCriticalSection pthread_mutex_unlock
#define stricmp strcasecmp
#else
#include "CapiHandler2.h"
#include "log.h"
#include "SimpleCapi.h"
#include <conio.h>
#include "wav2law.h"
#endif

#define CAPI ((CCapiHandler2*)(this->internal))
#define LOG ((((CCapiHandler2*)(this->internal))->log))

#ifdef LINUX
pthread_mutex_t line_mutex[64];
#else
CRITICAL_SECTION line_mutex[64];
#endif

int instancecount = 0;

static void readline(FILE*fi, char*line, int maxlen) {
    int pos = 0;
    while(!feof(fi)) {
        if(!fread(&line[pos],1,1,fi))
            break;
        if(line[pos] == 13 || line[pos]==10)
            break;
	if(pos<maxlen-1) 
	    pos++;
    }
    line[pos]=0;

    /* cut off comments */
    char*x=strchr(line,'#');
    if(x) {
        *x=0;
	pos = x-line;
    }

    /* cut off trailing whitespace */
    while(pos>=1 && (line[pos-1]==32 || line[pos-1]==9)) {
	pos--;
	line[pos]=0;
    }
}

static char* properties_msisdn = 0;
static char* properties_controller = 0; // all controllers

SimpleCapi::SimpleCapi(char*propertyfile, Logger*log) {
    char line[256]; // notice that line doesn't have anything to do with isdn, here.;)
    FILE*fi=fopen(propertyfile,"rb");

    if(!log) {
	Logger*log = new Logger();
	log->initLog (0,-1,0,0,-1,1); //-1)?
    }

    int properties_allcalls = 0;
    if(!fi) {
        log->logf("<warning> Couldn't open property file \"%s\". No MSISDN set!\n", propertyfile);
    }
    while(fi && !feof(fi)) {
        readline(fi,line, 256);
        char*a=strchr(line, '=');
        if(a) {
            *a=0;
            if (!stricmp(line, "local.msisdn"))
                properties_msisdn=strdup(a+1);
            else if (!stricmp(line, "local.allcalls"))
                properties_allcalls=atoi(a+1);
            else if (!stricmp(line, "local.controller"))
                properties_controller=strdup(a+1);
        }
    }
    if(fi)
	fclose(fi);

    this->internal = (void*)(new CCapiHandler2(log));
    ((CCapiHandler2*)(this->internal))->allcalls_hack = properties_allcalls;
    instancecount++;
    if(instancecount>=2) {
        LOG->logf("<error> Only one instance of SimpleCapi should be created.");
    }
}

SimpleCapi::~SimpleCapi() {
    LOG->log("<verbose> SimpleCapi::~SimpleCapi()");
    Logger*log = LOG;
    delete CAPI;
    delete log;
    instancecount--;
}

/* opens a new isdn channel, and returns an unique id for accessing it.
 *
 * @returns the line id
 */
int SimpleCapi::open (char*msisdn) {
    LOG->logf("<verbose> SimpleCapi::open(%s)",msisdn);
    if(!msisdn || !*msisdn) {
        msisdn = properties_msisdn;
	LOG->logf("<verbose> SimpleCapi::open(): using msisdn \"%s\" and controllermask [%s] from properties", msisdn,
		properties_controller?properties_controller:"all");
    } else {
	msisdn = strdup(msisdn);
    }
    EnterCriticalSection(&CAPI->capi_mutex); //probably not needed.
    int retval = CAPI->open(msisdn, properties_controller);
    LeaveCriticalSection(&CAPI->capi_mutex);
#ifndef LINUX
    if(retval>=0)
        InitializeCriticalSection(&line_mutex[retval]);
#endif
    LOG->logf("<verbose> SimpleCapi::open() returns line %d", retval);
    return retval;
}

/* closes an isdn channel, therefore giving it free.
 *
 * @param line the line id
 */
void SimpleCapi::close (int line) {
    LOG->logf("<verbose> SimpleCapi::close(%d)",line);
    if(!CAPI->checkLine(line))
        return;
    EnterCriticalSection(&line_mutex[line]);
    CAPI->close(line);
    LeaveCriticalSection(&line_mutex[line]);
#ifndef LINUX
    DeleteCriticalSection(&line_mutex[line]);
#endif
}

/*
 * This is the function to use if one want's to listen for calls. If
 * this is called, incoming connections will trigger a CAPI_EVENT_INCOMING
 * event. Note that to reset this, you'll have to close the line and
 * open another.
 * @param line the line to listen on
 */
void SimpleCapi::acceptCalls (int line) {
    LOG->logf("<verbose> SimpleCapi::acceptCalls(%d)",line);
    if(!CAPI->checkLine(line))
        return;
    EnterCriticalSection(&line_mutex[line]);
    CAPI->acceptCalls(line);
    LeaveCriticalSection(&line_mutex[line]);
}

/* call the specified msisdn.
 * 
 * @param line the line to use
 * @param msisdn the msisdn to call
 * @param timeout the number of seconds to wait for an answer
 * @returns 0 if nobody answered in time, >0 otherwise
 */
int SimpleCapi::call (int line, char* msisdn, int timeout) {
    LOG->logf("<verbose> SimpleCapi::call(%d, %s, %d)",line,msisdn,timeout);
    if(!CAPI->checkLine(line))
        return 0;
    EnterCriticalSection(&line_mutex[line]);
    CAPI->call (line, msisdn, -1);
    LeaveCriticalSection(&line_mutex[line]);

    float timeout2=(float)timeout;
    while(1) {
        EnterCriticalSection(&line_mutex[line]);
        ISDNEvent event = CAPI->listen(line, timeout2);
        LeaveCriticalSection(&line_mutex[line]);
        if(event.type == CAPI_EVENT_CONNECTED) {
            return 0;
        }
        /* CapiHandler2 will return an HANGUP event, even
         * if there hasn't been a connection yet. this means
         * for some reason we didn't manage to call the other
         * party
         */
        if(event.type == CAPI_EVENT_HANGUP) {
            if(event.reason)
                return event.reason;
            else
                return -1;
        }
        if(event.type == TIMEOUT) {
            return 1;
        }
    }
    return 1;
}

/* call the specified msisdn, return immediately. It returns
 * a job id which can be used to wait for the call to 
 * succeed 
 * @param line the line to use
 * @param msisdn the misdn to call
 * @returns a job id for the calling task
 */
int SimpleCapi::call_nonblock (int line, char* msisdn) {
    LOG->logf("<verbose> SimpleCapi::call_nonblock(%d, %s)",line, msisdn);
    if(!CAPI->checkLine(line))
        return -1;
    EnterCriticalSection(&line_mutex[line]);
    int retval  = CAPI->call (line, msisdn, -1);
    LeaveCriticalSection(&line_mutex[line]);

    return retval;
}

/* send a sound file into a line.  This call does not block,
 * but rather returns a job id which can be used
 * for waiting for the end of the sound file.
 *      @param line the line on which to play the sound file
 *	    @param filename the filename of the soundfile
 *	    @param mem the memory the sound file is in
 *	    @param len the length of the sound file
 *	    @returns a job id. An event with this job id will be
 *	             generated when the end of the audio data is 
 *	             reached.
 */
int SimpleCapi::play (int line, const char* filename) {
    FILE*fi = fopen (filename,"rb");
    if(!fi) {
        LOG->logf("<error> Could not find file\"%s\"!", filename);
        return -1;
    }
    fseek (fi, 0, SEEK_END);
    long int size = ftell(fi);
    fseek (fi, 0, SEEK_SET);

    LOG->logf ("<verbose> SimpleCapi::play(%d, %s) (%d bytes)", line, filename, size);

    char*data = (char*)malloc (size);
    fread (data, 1, size, fi);
    fclose (fi);
    int playjob = play ( line, data, size);
    free (data);
    return playjob;
}
int SimpleCapi::play (int line, void*mem, int len) {
    LOG->logf ("<verbose> SimpleCapi::play(%d, ptr, %d)", line,len);
    int riff = 0;
    unsigned char*mem2 = (unsigned char*)mem;
    if(mem2[0] == 0x52 && mem2[1] == 0x49 && mem2[2] == 0x46 && mem2[3] == 0x46) //RIFF
    {
        LOG->log ("<notice> Performing wav to law conversion");
        mem = wav2law(mem2, len, &len);
        riff = 1;
    }
    if(!CAPI->checkLine(line))
        return -1;
    EnterCriticalSection(&line_mutex[line]);
    int ret=CAPI->play (line, mem, len);
    LeaveCriticalSection(&line_mutex[line]);
    if(riff)
        free(mem);
    return ret;
}

/* record the other end's voice into buf. This call does not block,
 * but rather returns a job id which can be used for waiting for
 * the end of an recording. Notice that other events (especially
 * DIALTONE and HANGUP) will still be sent.
 *	    @param line the line id
 *	    @param buf the buffer to copy the audio data to
 *	    @param bufsize the maximal size of the audio data
 *	    @returns a job id, which can be used to wait for the end
 *	     of the recording
 */
int SimpleCapi::record (int line, void*buf, int bufsize) {
    LOG->logf ("<verbose> SimpleCapi::record(%d, ptr, %d)",line,bufsize);
    if(!CAPI->checkLine(line))
        return -1;
    EnterCriticalSection(&line_mutex[line]);
    int ret = CAPI->record(line,buf,bufsize);
    LeaveCriticalSection(&line_mutex[line]);

    return ret;
}

/* return the number of bytes read by last record command
 *      @param line the line id
 *      @returns number of bytes read
 */
int SimpleCapi::getRecordSize (int line) {
    EnterCriticalSection(&line_mutex[line]);
    int ret = CAPI->getRecordSize(line);
    LeaveCriticalSection(&line_mutex[line]);
    return ret;
}

/* send a fax through the line. This call blocks till the fax
 * has been transmitted.
 * @param line the line id
 * @param fax the fax data
 * @param faxsize the length of the fax data buffer
 */
int SimpleCapi::fax (int line, char*msisdn, char*filename) {
    int faxformat = 5; //ascii
    {
        FILE*fi= fopen(filename, "rb");
        char a[4];
        if(fi && fread(a,1,4,fi)==4) {
            if (!strncmp(a,"Sfff",4))
                faxformat = 0;
	    fclose(fi);
        }
    }
    printf("faxformat: %d\n", faxformat);

    LOG->logf ("<verbose> SimpleCapi::fax(%d, %s, %s) (faxformat = %d)", line,msisdn,filename, faxformat);
    EnterCriticalSection(&line_mutex[line]);
    if(!CAPI->checkLine(line))
        return -1;
    int ret =  CAPI->call (line, msisdn, faxformat);
    LeaveCriticalSection(&line_mutex[line]);
    if (!ret) {
        printf("no answer\n");
        return 0;
    }
    int done=0;
    while (!done) {
        ISDNEvent event = listen(line, 0);
        switch (event.type) {
        case CAPI_EVENT_CONNECTED:
            done = 1;
            break;
        default:
            printf ("Unexpected: %s\n", eventToString (&event));
            break;
        }
    }

    /* to "play" means to transfer data, which is exactly what we want to
     * do here */
    int playjob = play (line, filename);
    done=0;
    while (!done) {
        ISDNEvent event = listen(line, 0);
        switch (event.type) {
        case CAPI_EVENT_HANGUP:
            printf ("hung up\n");
            return 0;

        case CAPI_EVENT_DONE:
            if (event.jobid == playjob) {
                done = 1;
            }
            break;

        default:
            printf ("Unexpected: %s\n", eventToString (&event));
            break;
        }
    }

    done = 0;
    while(!done) {
        ISDNEvent event = listen(line, 1);
        if(event.type == CAPI_EVENT_HANGUP) {
            done=1;
        }
    }

    return 1;
}

/*
 * open a X31 connection to the specified msn.
 * @param line the line id
 * @param msn the msisdn of the other end
 * @param datex the is the x.25 address of the called party
 */
int SimpleCapi::callX31 (int line, char* msn, char* datex) {
    LOG->logf("<verbose> SimpleCapi::sendX31data(%d, %s)", line, msn);

    EnterCriticalSection(&line_mutex[line]);
    if (!CAPI->checkLine(line)) {
        return -1;
    }
    
    CAPI->setDatexAddress(line, datex);
    int ret = CAPI->call(line, msn, 100); // start X25 DTE-DTE call
    LeaveCriticalSection(&line_mutex[line]);
    if (!ret) {
        printf("no answer");
        return 0;
    }

    int done = 0;
    while (!done) {
        ISDNEvent event = listen(line, 0);
        switch (event.type) {
        case CAPI_EVENT_CONNECTED:
            done = 1;
            break;
        default:
            printf("Unexpected: %s\n", eventToString(&event));
	    return -2;
            break;
        }
    }
    return 1;
}

/* stop a running job. Most likely you may want to stop jobs
 * which play soundfiles once in a while for barge-in functionality 
 * @param jobid the job to stop
 */
void SimpleCapi::stop (int jobid) {
    LOG->logf("<verbose> SimpleCapi::stop(%d)", jobid);
    EnterCriticalSection(&CAPI->capi_mutex); //FIXME
    CAPI->stop(jobid);
    LeaveCriticalSection(&CAPI->capi_mutex);
}

/* wait for something to happen. This might be a hangup by the other
 * end, a job that finished, an incoming call etc.
 *	    @param line the line to listen on. If -1, listen on every line
 *	    @param timeout the time to listen. If nothing happened while
 *                we were waiting, return a TIMEOUT event. If timeout<0, 
 *                wait forever.
 */
ISDNEvent SimpleCapi::listen (int line, int timeout) {
    float timeout2 = (float)timeout;
    LOG->logf("<verbose> SimpleCapi::listen(%d, %f)", line, timeout2);
    ISDNEvent ret;
    ret.type = -1;
    if(!CAPI->checkLine(line))
        return ret;
    EnterCriticalSection(&line_mutex[line]);
    ret = CAPI->listen(line, timeout2);
    LeaveCriticalSection(&line_mutex[line]);
    return ret;
}

/* answer an incoming call.
 *	    @param event the INCOMING event to answer
 *	    @returns true on success, false on failure (e.g. if the caller
 *                hung up already)
 */
int SimpleCapi::answer (ISDNEvent*event) {
    int line = event->line;
    LOG->logf("<verbose> SimpleCapi::answer(%s) (line %d)", eventToString(event), line);
    if(!CAPI->checkLine(line))
        return 0;
    EnterCriticalSection(&line_mutex[line]);
    int ret = CAPI->answer(event);
    LeaveCriticalSection(&line_mutex[line]);
    return ret;
}

/* forward an incoming call / a connection
 *	    @param line the line to forward
 *	    @param msisdn the new msisdn
 *	    @returns true on success, false on failure (e.g. if the caller
 *                hung up already)
 */
int SimpleCapi::forward (int line, char*msisdn) {
    LOG->logf("<verbose> SimpleCapi::forward(%d, %s)", line, msisdn);
    if(!CAPI->checkLine(line))
        return 0;
    EnterCriticalSection(&line_mutex[line]);
    int ret;

    if(!strcmp(msisdn, "hold"))
        ret = CAPI->hold(line);
    else if(!strcmp(msisdn, "retrieve"))
        ret = CAPI->retrieve(line);
    else if(!strcmp(msisdn, "3pty"))
        ret = CAPI->conference(line);
    else if(!strcmp(msisdn, "ect"))
        ret = CAPI->ect(line);
    else
        ret = CAPI->forward(line, msisdn);

    LeaveCriticalSection(&line_mutex[line]);
    return ret;
}

/* hangup on a connection. This call blocks until the connection is correctly
 * terminated.
 * @param line the line id
 */
void SimpleCapi::hangup (int line) {
    LOG->logf ("<verbose> SimpleCapi::hangup(%d)", line);
    if(!CAPI->checkLine(line))
        return ;
    EnterCriticalSection(&line_mutex[line]);
    CAPI->hangup(line);
    LeaveCriticalSection(&line_mutex[line]);
}

/* convert an event to string. Useful for debugging.
 * @param event the Event to convert
 * @returns a newly allocated string. If you like your memorymanager, you give
 * it free after printing it.
 */
char* SimpleCapi::eventToString (ISDNEvent*event) {
    LOG->logf("<verbose> SimpleCapi::eventToString(line:%d)", event->line);
    char* ret = CAPI->eventToString(event);
    return ret;
}

